/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software; you can redistribute it and/or modify it under the terms of
the GNU General Public License, version 2, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

/// Handles all events from SDL_PollEvent.
module mde.events;

import global = mde.global;
static import mde.SDL;

import mde.input.input;
import mde.input.exception;

import mde.scheduler.InitFunctions;
import mde.scheduler.runTime;

import derelict.sdl.events;

import tango.util.log.Log : Log, Logger;

private Logger logger;
static this() {
    logger = Log.getLogger ("mde.events");
    
    init.addFunc (&initInput);

    Scheduler.perFrame (&mde.events.pollEvents);
}

void initInput () { // init func
    try {
        global.input = new Input();
        global.input.loadConfig ();         // (may also create instance)
    
        global.input.addButtonCallback (cast(Input.inputID) 0x0u, delegate void(Input.inputID i, bool b) {
            if (b) {
                logger.info ("Quiting...");
                global.run = false;
            }
        } );
    } catch (Exception) {
        setInitFailure;
    }
}

void pollEvents (double) {
    SDL_Event event;
    while (SDL_PollEvent (&event)) {
        switch (event.type) {
            case SDL_QUIT:
                logger.info ("Quit requested");
                global.run = false;
                break;
            case SDL_VIDEORESIZE:
                mde.SDL.resizeWindow (event.resize.w, event.resize.h);
                Scheduler.requestUpdate(RF_KEYS.DRAW);
                break;
            case SDL_ACTIVEEVENT:
            case SDL_VIDEOEXPOSE:
                Scheduler.requestUpdate(RF_KEYS.DRAW);
                break;
            default:
                try {
                    global.input (event);
                } catch (InputClassException e) {
                    logger.error ("Caught input exception; event will be ignored. Exception was:");
                    logger.error (e.msg);
                }
        }
    }
}
